* Mexican Stata Users Group Meeting, 2009
* Sophia Rabe-Hesketh
* "Multilevel Modeling of Ordinal Responses"

****** Estimation of proportional odds model *************************
* Recovery after surgery data
use http://www.stata-press.com/data/mlmus2/recovery, clear

* Prepare data for multilevel modeling
reshape long score, i(id dosage) j(occ)
egen id2 = group(id dosage)
recode score 0/1=1 2/3=2 4/5=3 6=4
recode occ 1=0 2=5 3=15 4=30, generate(time)

* single-level model (slide 16,17)
ologit score dosage age duration time, or

* basic two-level model (slide 16,17)
gllamm score dosage age duration time, ///
   i(id2) link(ologit) adapt
gllamm, eform
estimates store recovery1
* estimates save recovery1 /* for use in future session */

******* Relax proportional odds assumption (slide 22)*****************
* relax proportional odds assumption for dosage (slide 22)
eq thr: dosage
estimates restore recovery1
matrix a=e(b)
gllamm score age duration time, i(id2) ///
   link(ologit) thresh(thr) from(a) skip adapt
estimates store recovery2
* estimates save recovery2 /* for use in future session */

lrtest recovery1 recovery2

* Note: need to produce prediction dataset for graph
*       see below


******** Prediction of marginal and conditional prob. ****************
* predict marginal probabilities (slides 18,19)
estimates restore recovery1
predict p_marg, marg mu above(2) fsample
* Note: need to collapse data to average over age and
*       duration for graph on slide 19, see below

* predict posterior-mean probabilities (slides 18,20)
estimates restore recovery1
predict sp1, mu above(1) fsample
predict sp2, mu above(2) fsample
predict sp3, mu above(3) fsample

* graph of subject-specific probabilities (slide 20)
twoway (line sp1 time, sort lpat(dash) lw(medthick)) ///
       (line sp2 time, sort lpat(solid) lw(medthick)) ///
       (line sp3 time, sort lpat(shortdash) lw(medthick)) ///
  if dosage==20, by(id2) ///
  legend(order(1 "above(1)" 2 "above(2)" 3 "above(3)") cols(3)) ///
  xtitle(time)

* predicted probabilities for model that relaxes proportional odds
estimates restore recovery2
predict prob2, marg mu above(2) fsample

*********** Average over age and duration (slide 21) *****************

* Average over age and duration to compare observed with 
* predicted probabilities P(y>2|dosage,time) --> slide 21

gen one=1
gen above2 = score>2
collapse (sum) num=one (mean) p_marg prob2 above2 age duration, ///
    by(dosage time)

* graph for slide 21
twoway (line above2 time, sort lw(medthick)) ///
   (line p_marg time, sort lpat(dash) lw(medthick)), by(dosage) ///
   legend(order(1 "observed" 2 "predicted")) ///
   xtitle(time)

********** Create prediction dataset (slide 19)***********************
* need to have all variables that were in model
gen id2=1
gen score=1
* set age to 37 and duration to 80
replace age=37
replace duration=80

estimates restore recovery1
gllapred pr1, marg mu above(1) fsample
gllapred pr2, marg mu above(2) fsample
gllapred pr3, marg mu above(3) fsample

* graph for slide 19
twoway (line pr1 time, sort lpat(dash) lw(medthick)) ///
       (line pr2 time, sort lpat(solid) lw(medthick)) ///
       (line pr3 time, sort lpat(shortdash) lw(medthick)), ///
  by(dosage) ///
  legend(order(1 "above(1)" 2 "above(2)" 3 "above(3)") cols(3)) ///
  xtitle(time)

* logits for slide 23
* proportional odds model
gen lg1 = logit(pr1)
gen lg2 = logit(pr2)
gen lg3 = logit(pr3)

* for non-proportional odds model
estimates restore recovery2
gllapred pr12, marg mu above(1) fsample
gllapred pr22, marg mu above(2) fsample
gllapred pr32, marg mu above(3) fsample

gen lg12 = logit(pr12)
gen lg22 = logit(pr22)
gen lg32 = logit(pr32)

* make tick mark labels for logit scale (y-axis) on slide 23
* see Nick Cox <SJ 8(1):142--145> 
global label
foreach n of num 50(10)90 95 98 {
 * disp `n'
 local ll = logit(`n'/100)
 global label $label `ll' ".`n'"
}

* graphs for slide 23
twoway (line lg12 dosage, lpat(dash) lw(thick)) ///
   (line lg22 dosage, lpat(solid) lw(thick)) ///
   (line lg32 dosage, lpat(shortdash) lw(thick)) if time==30, ///
   ylabel($label, ang(h)) name(notpar) ///
   legend(order(1 "above(1)" 2 "above(2)" 3 "above(3)") cols(3)) 


twoway (line lg1 dosage, lpat(dash) lw(thick)) ///
   (line lg2 dosage, lpat(solid) lw(thick)) ///
   (line lg3 dosage, lpat(shortdash) lw(thick)) if time==30, ///
   ylabel($label, ang(h))  ///
   legend(order(1 "above(1)" 2 "above(2)" 3 "above(3)") cols(3)) 


*********** Relax homoscedasticity ***********************************

* read and prepare data again
use http://www.stata-press.com/data/mlmus2/recovery, clear
reshape long score, i(id dosage) j(occ)
egen id2 = group(id dosage)
recode score 0/1=1 2/3=2 4/5=3 6=4
recode occ 1=0 2=5 3=15 4=30, generate(time)

* single-level model
oprobit score dosage age duration time

* basic two-level model
gllamm score dosage age duration time, i(id2) ///
   link(oprobit) adapt
estimates store probit1

* heteroscedasticity at level 1 (slide 24)
tab dosage, gen(d)
eq het: d2 d3 d4
matrix a=e(b)
gllamm score dosage age duration time, i(id2) ///
   link(soprobit) s(het) from(a) adapt
lrtest probit1 .

* heteroscedasticity at level 2 (slide 25)
eq load: d1 d2 d3 d4
gllamm score dosage age duration time, i(id2) ///
   eqs(load) link(oprobit) adapt
lrtest probit1 .

